#ifndef LSTMNETWORK__H
#define LSTMNETWORK__H

#include "NeuralNetwork.hpp"
#include "ContextLSTM.hpp"
#include "RawInputLSTM.hpp"
#include "SplitTransLSTM.hpp"
#include "FocusedColumnLSTM.hpp"
#include "MLP.hpp"
#include "DepthLayerTreeEmbedding.hpp"

class LSTMNetworkImpl : public NeuralNetworkImpl
{
  private :

  torch::nn::Embedding wordEmbeddings{nullptr};
  torch::nn::Dropout embeddingsDropout{nullptr};

  MLP mlp{nullptr};
  ContextLSTM contextLSTM{nullptr};
  RawInputLSTM rawInputLSTM{nullptr};
  SplitTransLSTM splitTransLSTM{nullptr};
  DepthLayerTreeEmbedding treeEmbedding{nullptr};
  std::vector<FocusedColumnLSTM> focusedLstms;

  bool hasRawInputLSTM{false};
  bool hasTreeEmbedding{false};

  public :

  LSTMNetworkImpl(int nbOutputs, int unknownValueThreshold, std::vector<int> bufferContext, std::vector<int> stackContext, std::vector<std::string> columns, std::vector<int> focusedBufferIndexes, std::vector<int> focusedStackIndexes, std::vector<std::string> focusedColumns, std::vector<int> maxNbElements, int leftWindowRawInput, int rightWindowRawInput, int embeddingsSize, std::vector<std::pair<int, float>> mlpParams, int contextLSTMSize, int focusedLSTMSize, int rawInputLSTMSize, int splitTransLSTMSize, int numLayers, bool bilstm, float lstmDropout, std::vector<std::string> treeEmbeddingColumns, std::vector<int> treeEmbeddingBuffer, std::vector<int> treeEmbeddingStack, std::vector<int> treeEmbeddingNbElems, int treeEmbeddingSize);
  torch::Tensor forward(torch::Tensor input) override;
  std::vector<std::vector<long>> extractContext(Config & config, Dict & dict) const override;
};

#endif
